/*-
 * Copyright (c) 2018 UPLEX Nils Goroll Systemoptimierung
 * All rights reserved
 *
 * Author: Geoffrey Simmons <geoffrey.simmons@uplex.de>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package log

import (
	"bytes"
	"strconv"
	"testing"
)

type atoUint32Exp struct {
	val uint32
	ok  bool
}

func TestAtoUint32(t *testing.T) {
	expMap := map[string]atoUint32Exp{
		"":           atoUint32Exp{val: 0, ok: false},
		"0":          atoUint32Exp{val: 0, ok: true},
		"1":          atoUint32Exp{val: 1, ok: true},
		"9":          atoUint32Exp{val: 9, ok: true},
		"10":         atoUint32Exp{val: 10, ok: true},
		"101":        atoUint32Exp{val: 101, ok: true},
		"1009":       atoUint32Exp{val: 1009, ok: true},
		"foo":        atoUint32Exp{val: 0, ok: false},
		"4294967290": atoUint32Exp{val: 4294967290, ok: true},
		"4294967291": atoUint32Exp{val: 4294967291, ok: true},
		"4294967292": atoUint32Exp{val: 4294967292, ok: true},
		"4294967293": atoUint32Exp{val: 4294967293, ok: true},
		"4294967294": atoUint32Exp{val: 4294967294, ok: true},
		"4294967295": atoUint32Exp{val: 4294967295, ok: true},
		"4294967296": atoUint32Exp{val: 0, ok: false},
		"4294967297": atoUint32Exp{val: 0, ok: false},
		"4294967298": atoUint32Exp{val: 0, ok: false},
		"4294967299": atoUint32Exp{val: 0, ok: false},
	}

	for str, exp := range expMap {
		val, ok := atoUint32([]byte(str))
		if val != exp.val || ok != exp.ok {
			t.Errorf("atoUint32(%s) want=%v got=%v", str, exp,
				atoUint32Exp{val: val, ok: ok})
		}
	}
}

func BenchmarkAtoUint32(b *testing.B) {
	bytes := []byte("4294967295")
	b.ReportAllocs()
	b.ResetTimer()
	for i := 0; i < b.N; i++ {
		_, ok := atoUint32(bytes)
		if !ok {
			b.Fatal("atoUint32() failed to parse")
			return
		}
	}
}

func BenchmarkStrconvAtoi(b *testing.B) {
	bytes := []byte("4294967295")
	b.ReportAllocs()
	b.ResetTimer()
	for i := 0; i < b.N; i++ {
		_, err := strconv.Atoi(string(bytes))
		if err != nil {
			b.Fatal("strconv.Atoi() failed to parse")
			return
		}
	}
}

var delimVec = map[string][]struct {
	fld int
	ok  bool
	exp string
}{
	"foo bar baz quux": {
		{0, true, "foo"},
		{1, true, "bar"},
		{2, true, "baz"},
		{3, true, "quux"},
		{4, false, ""},
		{5, false, ""},
	},
	"  foo \t \n  bar\r  \t   baz    quux\n\r\n\n": {
		{0, true, "foo"},
		{1, true, "bar"},
		{2, true, "baz"},
		{3, true, "quux"},
		{4, false, ""},
		{5, false, ""},
	},
	"foobarbazquux": {
		{0, true, "foobarbazquux"},
		{1, false, ""},
		{2, false, ""},
	},
	" \t\n\r foobarbazquux\r\n\n\n": {
		{0, true, "foobarbazquux"},
		{1, false, ""},
		{2, false, ""},
	},
	"foobarbazquux     ": {
		{0, true, "foobarbazquux"},
		{1, false, ""},
		{2, false, ""},
	},
	"     foobarbazquux": {
		{0, true, "foobarbazquux"},
		{1, false, ""},
		{2, false, ""},
	},
	"": {
		{0, false, ""},
		{1, false, ""},
		{2, false, ""},
	},
}

func TestFieldNDelims(t *testing.T) {
	s := []byte("foo bar baz quux")
	_, _, ok := fieldNDelims(s, 4)
	if ok {
		t.Error("fieldNDelims() did not fail for N out of range")
	}
	_, _, ok = fieldNDelims(s, -1)
	if ok {
		t.Error("fieldNDelims() did not fail for N < 0")
	}
	_, _, ok = fieldNDelims(nil, 0)
	if ok {
		t.Error("fieldNDelims() did not fail the nil slice")
	}

	for str, expVec := range delimVec {
		sl := []byte(str)
		for _, exp := range expVec {
			min, max, ok := fieldNDelims(sl, exp.fld)
			if ok != exp.ok {
				t.Errorf("fieldNdelims(%s, %d) ok want=%v "+
					"got=%v", str, exp.fld, exp.ok, ok)
			}
			if exp.ok {
				f := string(sl[min:max])
				if f != exp.exp {
					t.Errorf("fieldNDelims(%s, %d) want=%v"+
						" got=%v", str, exp.fld,
						exp.exp, f)
				}
			}
		}
	}
}

func BenchmarkFieldNDelims(b *testing.B) {
	bytes := []byte("foo bar baz quux")
	b.ReportAllocs()
	b.ResetTimer()
	for i := 0; i < b.N; i++ {
		_, _, ok := fieldNDelims(bytes, 3)
		if !ok {
			b.Fatal("fieldNDelims() failed to parse")
			return
		}
	}
}

func BenchmarkBytesFields(b *testing.B) {
	s := []byte("foo bar baz quux")
	b.ReportAllocs()
	b.ResetTimer()
	for i := 0; i < b.N; i++ {
		_ = bytes.Fields(s)
	}
}

type parseInt64Exp struct {
	val int64
	ok  bool
}

func TestParseInt64(t *testing.T) {
	expMap := map[string]parseInt64Exp{
		"":            parseInt64Exp{val: 0, ok: false},
		"0":           parseInt64Exp{val: 0, ok: true},
		"1":           parseInt64Exp{val: 1, ok: true},
		"9":           parseInt64Exp{val: 9, ok: true},
		"10":          parseInt64Exp{val: 10, ok: true},
		"101":         parseInt64Exp{val: 101, ok: true},
		"1009":        parseInt64Exp{val: 1009, ok: true},
		"foo":         parseInt64Exp{val: 0, ok: false},
		" foo ":       parseInt64Exp{val: 0, ok: false},
		"foo  ":       parseInt64Exp{val: 0, ok: false},
		"  foo":       parseInt64Exp{val: 0, ok: false},
		"4294967290":  parseInt64Exp{val: 4294967290, ok: true},
		"4294967291":  parseInt64Exp{val: 4294967291, ok: true},
		"4294967292":  parseInt64Exp{val: 4294967292, ok: true},
		"4294967293":  parseInt64Exp{val: 4294967293, ok: true},
		"4294967294":  parseInt64Exp{val: 4294967294, ok: true},
		"4294967295":  parseInt64Exp{val: 4294967295, ok: true},
		"4294967296":  parseInt64Exp{val: 4294967296, ok: true},
		"4294967297":  parseInt64Exp{val: 4294967297, ok: true},
		"4294967298":  parseInt64Exp{val: 4294967298, ok: true},
		"4294967299":  parseInt64Exp{val: 4294967299, ok: true},
		"+0":          parseInt64Exp{val: 0, ok: true},
		"+1":          parseInt64Exp{val: 1, ok: true},
		"+9":          parseInt64Exp{val: 9, ok: true},
		"+10":         parseInt64Exp{val: 10, ok: true},
		"+101":        parseInt64Exp{val: 101, ok: true},
		"+1009":       parseInt64Exp{val: 1009, ok: true},
		"+4294967299": parseInt64Exp{val: 4294967299, ok: true},
		"-0":          parseInt64Exp{val: 0, ok: true},
		"-1":          parseInt64Exp{val: -1, ok: true},
		"-9":          parseInt64Exp{val: -9, ok: true},
		"-10":         parseInt64Exp{val: -10, ok: true},
		"-101":        parseInt64Exp{val: -101, ok: true},
		"-1009":       parseInt64Exp{val: -1009, ok: true},
		"-4294967299": parseInt64Exp{val: -4294967299, ok: true},
		" +1":         parseInt64Exp{val: 1, ok: true},
		" -1":         parseInt64Exp{val: -1, ok: true},
		"00":          parseInt64Exp{val: 0, ok: true},
		"01":          parseInt64Exp{val: 1, ok: true},
		"02":          parseInt64Exp{val: 2, ok: true},
		"03":          parseInt64Exp{val: 3, ok: true},
		"04":          parseInt64Exp{val: 4, ok: true},
		"05":          parseInt64Exp{val: 5, ok: true},
		"06":          parseInt64Exp{val: 6, ok: true},
		"07":          parseInt64Exp{val: 7, ok: true},
		"08":          parseInt64Exp{val: 0, ok: false},
		"09":          parseInt64Exp{val: 0, ok: false},
		"010":         parseInt64Exp{val: 8, ok: true},
		"011":         parseInt64Exp{val: 9, ok: true},
		"012":         parseInt64Exp{val: 10, ok: true},
		"013":         parseInt64Exp{val: 11, ok: true},
		"014":         parseInt64Exp{val: 12, ok: true},
		"015":         parseInt64Exp{val: 13, ok: true},
		"016":         parseInt64Exp{val: 14, ok: true},
		"017":         parseInt64Exp{val: 15, ok: true},
		"0100":        parseInt64Exp{val: 64, ok: true},
		"0177":        parseInt64Exp{val: 127, ok: true},
		"01000":       parseInt64Exp{val: 512, ok: true},
		"01777":       parseInt64Exp{val: 1023, ok: true},
		"0187":        parseInt64Exp{val: 0, ok: false},
		"+00":         parseInt64Exp{val: 0, ok: true},
		"+01":         parseInt64Exp{val: 1, ok: true},
		"+02":         parseInt64Exp{val: 2, ok: true},
		"+03":         parseInt64Exp{val: 3, ok: true},
		"+04":         parseInt64Exp{val: 4, ok: true},
		"+05":         parseInt64Exp{val: 5, ok: true},
		"+06":         parseInt64Exp{val: 6, ok: true},
		"+07":         parseInt64Exp{val: 7, ok: true},
		"+08":         parseInt64Exp{val: 0, ok: false},
		"+09":         parseInt64Exp{val: 0, ok: false},
		"+010":        parseInt64Exp{val: 8, ok: true},
		"+011":        parseInt64Exp{val: 9, ok: true},
		"+012":        parseInt64Exp{val: 10, ok: true},
		"+013":        parseInt64Exp{val: 11, ok: true},
		"+014":        parseInt64Exp{val: 12, ok: true},
		"+015":        parseInt64Exp{val: 13, ok: true},
		"+016":        parseInt64Exp{val: 14, ok: true},
		"+017":        parseInt64Exp{val: 15, ok: true},
		"+0100":       parseInt64Exp{val: 64, ok: true},
		"+0177":       parseInt64Exp{val: 127, ok: true},
		"+01000":      parseInt64Exp{val: 512, ok: true},
		"+01777":      parseInt64Exp{val: 1023, ok: true},
		"-00":         parseInt64Exp{val: 0, ok: true},
		"-01":         parseInt64Exp{val: -1, ok: true},
		"-02":         parseInt64Exp{val: -2, ok: true},
		"-03":         parseInt64Exp{val: -3, ok: true},
		"-04":         parseInt64Exp{val: -4, ok: true},
		"-05":         parseInt64Exp{val: -5, ok: true},
		"-06":         parseInt64Exp{val: -6, ok: true},
		"-07":         parseInt64Exp{val: -7, ok: true},
		"-08":         parseInt64Exp{val: 0, ok: false},
		"-09":         parseInt64Exp{val: 0, ok: false},
		"-010":        parseInt64Exp{val: -8, ok: true},
		"-011":        parseInt64Exp{val: -9, ok: true},
		"-012":        parseInt64Exp{val: -10, ok: true},
		"-013":        parseInt64Exp{val: -11, ok: true},
		"-014":        parseInt64Exp{val: -12, ok: true},
		"-015":        parseInt64Exp{val: -13, ok: true},
		"-016":        parseInt64Exp{val: -14, ok: true},
		"-017":        parseInt64Exp{val: -15, ok: true},
		"-0100":       parseInt64Exp{val: -64, ok: true},
		"-0177":       parseInt64Exp{val: -127, ok: true},
		"-01000":      parseInt64Exp{val: -512, ok: true},
		"-01777":      parseInt64Exp{val: -1023, ok: true},
		" 01777 ":     parseInt64Exp{val: 1023, ok: true},
		" +01777 ":    parseInt64Exp{val: 1023, ok: true},
		" -01777 ":    parseInt64Exp{val: -1023, ok: true},
		"0x0":         parseInt64Exp{val: 0, ok: true},
		"0x1":         parseInt64Exp{val: 1, ok: true},
		"0x2":         parseInt64Exp{val: 2, ok: true},
		"0x3":         parseInt64Exp{val: 3, ok: true},
		"0x4":         parseInt64Exp{val: 4, ok: true},
		"0x5":         parseInt64Exp{val: 5, ok: true},
		"0x6":         parseInt64Exp{val: 6, ok: true},
		"0x7":         parseInt64Exp{val: 7, ok: true},
		"0x8":         parseInt64Exp{val: 8, ok: true},
		"0x9":         parseInt64Exp{val: 9, ok: true},
		"0xa":         parseInt64Exp{val: 10, ok: true},
		"0xb":         parseInt64Exp{val: 11, ok: true},
		"0xc":         parseInt64Exp{val: 12, ok: true},
		"0xd":         parseInt64Exp{val: 13, ok: true},
		"0xe":         parseInt64Exp{val: 14, ok: true},
		"0xf":         parseInt64Exp{val: 15, ok: true},
		"0xA":         parseInt64Exp{val: 10, ok: true},
		"0xB":         parseInt64Exp{val: 11, ok: true},
		"0xC":         parseInt64Exp{val: 12, ok: true},
		"0xD":         parseInt64Exp{val: 13, ok: true},
		"0xE":         parseInt64Exp{val: 14, ok: true},
		"0xF":         parseInt64Exp{val: 15, ok: true},
		"0xg":         parseInt64Exp{val: 0, ok: false},
		"0xG":         parseInt64Exp{val: 0, ok: false},
		"0x10":        parseInt64Exp{val: 16, ok: true},
		"0x11":        parseInt64Exp{val: 17, ok: true},
		"0x12":        parseInt64Exp{val: 18, ok: true},
		"0x100":       parseInt64Exp{val: 256, ok: true},
		"0x1fF":       parseInt64Exp{val: 511, ok: true},
		"0xdeadbeef":  parseInt64Exp{val: 3735928559, ok: true},
		"0xDeAdBeEf":  parseInt64Exp{val: 3735928559, ok: true},
		"0x1gF":       parseInt64Exp{val: 0, ok: false},
		"0x1=F":       parseInt64Exp{val: 0, ok: false},
		"+0x0":        parseInt64Exp{val: 0, ok: true},
		"+0x1":        parseInt64Exp{val: 1, ok: true},
		"+0x2":        parseInt64Exp{val: 2, ok: true},
		"+0x3":        parseInt64Exp{val: 3, ok: true},
		"+0x4":        parseInt64Exp{val: 4, ok: true},
		"+0x5":        parseInt64Exp{val: 5, ok: true},
		"+0x6":        parseInt64Exp{val: 6, ok: true},
		"+0x7":        parseInt64Exp{val: 7, ok: true},
		"+0x8":        parseInt64Exp{val: 8, ok: true},
		"+0x9":        parseInt64Exp{val: 9, ok: true},
		"+0xa":        parseInt64Exp{val: 10, ok: true},
		"+0xb":        parseInt64Exp{val: 11, ok: true},
		"+0xc":        parseInt64Exp{val: 12, ok: true},
		"+0xd":        parseInt64Exp{val: 13, ok: true},
		"+0xe":        parseInt64Exp{val: 14, ok: true},
		"+0xf":        parseInt64Exp{val: 15, ok: true},
		"+0xA":        parseInt64Exp{val: 10, ok: true},
		"+0xB":        parseInt64Exp{val: 11, ok: true},
		"+0xC":        parseInt64Exp{val: 12, ok: true},
		"+0xD":        parseInt64Exp{val: 13, ok: true},
		"+0xE":        parseInt64Exp{val: 14, ok: true},
		"+0xF":        parseInt64Exp{val: 15, ok: true},
		"+0x10":       parseInt64Exp{val: 16, ok: true},
		"+0x11":       parseInt64Exp{val: 17, ok: true},
		"+0x12":       parseInt64Exp{val: 18, ok: true},
		"+0x100":      parseInt64Exp{val: 256, ok: true},
		"+0x1fF":      parseInt64Exp{val: 511, ok: true},
		"+0xdeadbeef": parseInt64Exp{val: 3735928559, ok: true},
		"+0xDeAdBeEf": parseInt64Exp{val: 3735928559, ok: true},
		"-0x0":        parseInt64Exp{val: 0, ok: true},
		"-0x1":        parseInt64Exp{val: -1, ok: true},
		"-0x2":        parseInt64Exp{val: -2, ok: true},
		"-0x3":        parseInt64Exp{val: -3, ok: true},
		"-0x4":        parseInt64Exp{val: -4, ok: true},
		"-0x5":        parseInt64Exp{val: -5, ok: true},
		"-0x6":        parseInt64Exp{val: -6, ok: true},
		"-0x7":        parseInt64Exp{val: -7, ok: true},
		"-0x8":        parseInt64Exp{val: -8, ok: true},
		"-0x9":        parseInt64Exp{val: -9, ok: true},
		"-0xa":        parseInt64Exp{val: -10, ok: true},
		"-0xb":        parseInt64Exp{val: -11, ok: true},
		"-0xc":        parseInt64Exp{val: -12, ok: true},
		"-0xd":        parseInt64Exp{val: -13, ok: true},
		"-0xe":        parseInt64Exp{val: -14, ok: true},
		"-0xf":        parseInt64Exp{val: -15, ok: true},
		"-0xA":        parseInt64Exp{val: -10, ok: true},
		"-0xB":        parseInt64Exp{val: -11, ok: true},
		"-0xC":        parseInt64Exp{val: -12, ok: true},
		"-0xD":        parseInt64Exp{val: -13, ok: true},
		"-0xE":        parseInt64Exp{val: -14, ok: true},
		"-0xF":        parseInt64Exp{val: -15, ok: true},
		"-0x10":       parseInt64Exp{val: -16, ok: true},
		"-0x11":       parseInt64Exp{val: -17, ok: true},
		"-0x12":       parseInt64Exp{val: -18, ok: true},
		"-0x100":      parseInt64Exp{val: -256, ok: true},
		"-0x1fF":      parseInt64Exp{val: -511, ok: true},
		"-0xdeadbeef": parseInt64Exp{val: -3735928559, ok: true},
		"-0xDeAdBeEf": parseInt64Exp{val: -3735928559, ok: true},
		" +0x1 ":      parseInt64Exp{val: 1, ok: true},
		" -0x1 ":      parseInt64Exp{val: -1, ok: true},
		"0x1 ":        parseInt64Exp{val: 1, ok: true},
		" 0x1":        parseInt64Exp{val: 1, ok: true},
		" 1 2 ":       parseInt64Exp{val: 1, ok: true},
		" 1 foo ":     parseInt64Exp{val: 1, ok: true},
		" 1foo ":      parseInt64Exp{val: 0, ok: false},
		"9223372036854775800": parseInt64Exp{val: 9223372036854775800,
			ok: true},
		"9223372036854775801": parseInt64Exp{val: 9223372036854775801,
			ok: true},
		"9223372036854775802": parseInt64Exp{val: 9223372036854775802,
			ok: true},
		"9223372036854775803": parseInt64Exp{val: 9223372036854775803,
			ok: true},
		"9223372036854775804": parseInt64Exp{val: 9223372036854775804,
			ok: true},
		"9223372036854775805": parseInt64Exp{val: 9223372036854775805,
			ok: true},
		"9223372036854775806": parseInt64Exp{val: 9223372036854775806,
			ok: true},
		"9223372036854775807": parseInt64Exp{val: 9223372036854775807,
			ok: true},
		"9223372036854775808": parseInt64Exp{val: 0, ok: false},
		"9223372036854775809": parseInt64Exp{val: 0, ok: false},
		"9223372036854775810": parseInt64Exp{val: 0, ok: false},
		"9223372036854775811": parseInt64Exp{val: 0, ok: false},
		"+9223372036854775800": parseInt64Exp{val: 9223372036854775800,
			ok: true},
		"+9223372036854775801": parseInt64Exp{val: 9223372036854775801,
			ok: true},
		"+9223372036854775802": parseInt64Exp{val: 9223372036854775802,
			ok: true},
		"+9223372036854775803": parseInt64Exp{val: 9223372036854775803,
			ok: true},
		"+9223372036854775804": parseInt64Exp{val: 9223372036854775804,
			ok: true},
		"+9223372036854775805": parseInt64Exp{val: 9223372036854775805,
			ok: true},
		"+9223372036854775806": parseInt64Exp{val: 9223372036854775806,
			ok: true},
		"+9223372036854775807": parseInt64Exp{val: 9223372036854775807,
			ok: true},
		"+9223372036854775808": parseInt64Exp{val: 0, ok: false},
		"+9223372036854775809": parseInt64Exp{val: 0, ok: false},
		"+9223372036854775810": parseInt64Exp{val: 0, ok: false},
		"+9223372036854775811": parseInt64Exp{val: 0, ok: false},
		"-9223372036854775800": parseInt64Exp{val: -9223372036854775800,
			ok: true},
		"-9223372036854775801": parseInt64Exp{val: -9223372036854775801,
			ok: true},
		"-9223372036854775802": parseInt64Exp{val: -9223372036854775802,
			ok: true},
		"-9223372036854775803": parseInt64Exp{val: -9223372036854775803,
			ok: true},
		"-9223372036854775804": parseInt64Exp{val: -9223372036854775804,
			ok: true},
		"-9223372036854775805": parseInt64Exp{val: -9223372036854775805,
			ok: true},
		"-9223372036854775806": parseInt64Exp{val: -9223372036854775806,
			ok: true},
		"-9223372036854775807": parseInt64Exp{val: -9223372036854775807,
			ok: true},
		"-9223372036854775808": parseInt64Exp{val: -9223372036854775808,
			ok: true},
		"-9223372036854775809": parseInt64Exp{val: 0, ok: false},
		"-9223372036854775810": parseInt64Exp{val: 0, ok: false},
		"-9223372036854775820": parseInt64Exp{val: 0, ok: false},
		"0777777777777777777770": parseInt64Exp{val: 9223372036854775800,
			ok: true},
		"0777777777777777777771": parseInt64Exp{val: 9223372036854775801,
			ok: true},
		"0777777777777777777772": parseInt64Exp{val: 9223372036854775802,
			ok: true},
		"0777777777777777777773": parseInt64Exp{val: 9223372036854775803,
			ok: true},
		"0777777777777777777774": parseInt64Exp{val: 9223372036854775804,
			ok: true},
		"0777777777777777777775": parseInt64Exp{val: 9223372036854775805,
			ok: true},
		"0777777777777777777776": parseInt64Exp{val: 9223372036854775806,
			ok: true},
		"0777777777777777777777": parseInt64Exp{val: 9223372036854775807,
			ok: true},
		"01000000000000000000000": parseInt64Exp{val: 0, ok: false},
		"01000000000000000000001": parseInt64Exp{val: 0, ok: false},
		"01000000000000000000002": parseInt64Exp{val: 0, ok: false},
		"-0777777777777777777777": parseInt64Exp{val: -9223372036854775807,
			ok: true},
		"-01000000000000000000000": parseInt64Exp{val: -9223372036854775808,
			ok: true},
		"-01000000000000000000001": parseInt64Exp{val: 0, ok: false},
		"-01000000000000000000002": parseInt64Exp{val: 0, ok: false},
		"-01000000000000000000003": parseInt64Exp{val: 0, ok: false},
		"0x7FFFFFFFFFFFFFF0": parseInt64Exp{val: 9223372036854775792,
			ok: true},
		"0x7FFFFFFFFFFFFFF1": parseInt64Exp{val: 9223372036854775793,
			ok: true},
		"0x7FFFFFFFFFFFFFF2": parseInt64Exp{val: 9223372036854775794,
			ok: true},
		"0x7FFFFFFFFFFFFFF3": parseInt64Exp{val: 9223372036854775795,
			ok: true},
		"0x7FFFFFFFFFFFFFF4": parseInt64Exp{val: 9223372036854775796,
			ok: true},
		"0x7FFFFFFFFFFFFFF5": parseInt64Exp{val: 9223372036854775797,
			ok: true},
		"0x7FFFFFFFFFFFFFF6": parseInt64Exp{val: 9223372036854775798,
			ok: true},
		"0x7FFFFFFFFFFFFFF7": parseInt64Exp{val: 9223372036854775799,
			ok: true},
		"0x7FFFFFFFFFFFFFF8": parseInt64Exp{val: 9223372036854775800,
			ok: true},
		"0x7FFFFFFFFFFFFFF9": parseInt64Exp{val: 9223372036854775801,
			ok: true},
		"0x7FFFFFFFFFFFFFFA": parseInt64Exp{val: 9223372036854775802,
			ok: true},
		"0x7FFFFFFFFFFFFFFB": parseInt64Exp{val: 9223372036854775803,
			ok: true},
		"0x7FFFFFFFFFFFFFFC": parseInt64Exp{val: 9223372036854775804,
			ok: true},
		"0x7FFFFFFFFFFFFFFD": parseInt64Exp{val: 9223372036854775805,
			ok: true},
		"0x7FFFFFFFFFFFFFFE": parseInt64Exp{val: 9223372036854775806,
			ok: true},
		"0x7FFFFFFFFFFFFFFF": parseInt64Exp{val: 9223372036854775807,
			ok: true},
		"0x8000000000000000": parseInt64Exp{val: 0, ok: false},
		"0x8000000000000001": parseInt64Exp{val: 0, ok: false},
		"0x8000000000000002": parseInt64Exp{val: 0, ok: false},
		"-0x7FFFFFFFFFFFFFFE": parseInt64Exp{val: -9223372036854775806,
			ok: true},
		"-0x7FFFFFFFFFFFFFFF": parseInt64Exp{val: -9223372036854775807,
			ok: true},
		"-0x8000000000000000": parseInt64Exp{val: -9223372036854775808,
			ok: true},
		"-0x8000000000000001": parseInt64Exp{val: 0, ok: false},
		"-0x8000000000000002": parseInt64Exp{val: 0, ok: false},
	}

	for str, exp := range expMap {
		val, ok := parseInt64([]byte(str))
		if val != exp.val || ok != exp.ok {
			t.Errorf("parseInt64(%s) want=%v got=%v", str, exp,
				parseInt64Exp{val: val, ok: ok})
		}
	}
}

func BenchmarkParseInt64(b *testing.B) {
	s := []byte("9223372036854775807")
	b.ReportAllocs()
	b.ResetTimer()
	for i := 0; i < b.N; i++ {
		_, ok := parseInt64(s)
		if !ok {
			b.Fatal("parseInt64() failed to parse")
			return
		}
	}
}

func BenchmarkStrconvParseInt(b *testing.B) {
	s := []byte("9223372036854775807")
	b.ReportAllocs()
	b.ResetTimer()
	for i := 0; i < b.N; i++ {
		_, _ = strconv.ParseInt(string(s), 0, 64)
	}
}
